require "jobparser/version"
require "jobparser/parser"
require "jobparser/parsehtml"
require "jobparser/parseschema"
require "jobparser/cleaner"
require "jobparser/scorer"
require "jobparser/specialcases"
require "jobparser/cache"
require "jobparser/cache/textfile"
require "jobparser/cache/mongostore"
require "jobparser/facets/facet"
require "jobparser/facets/salary"
require "jobparser/facets/salarystring"
require "jobparser/facets/location"
require "jobparser/facets/apply"
require "jobparser/facets/title"
require "jobparser/facets/deadline"
require "jobparser/facets/postcode"
require "jobparser/error"

require "uri"
require "open-uri"
require "open_uri_redirections"

module JobParser
  def self.parser(url)
    puts "Warning: JobParser.parser is old. Use JobParser.parse"
    JobParser.parse(url)
  end

  def self.parse(url)
    if JobParser.cache.valid_for_url?(url)
      res = JobParser.cache.fetch_result_for_url(url)
      res[:schema] ? ParseSchema.new(nil, url) : ParseHtml.new(nil, url)
    else
      begin
        html = open(URI.encode(url), :allow_redirections => :safe).read
        if html.include?("http://schema.org/JobPosting")
          ParseSchema.new(html, url)
        else
          ParseHtml.new(html, url)
        end
      rescue URI::InvalidURIError
        raise JobParser::Error::InvalidUrl, "The URI given (\"#{url}\") was not valid"
      end
    end
  end

  def self.config
    @config
  end

  def self.cache
    @cache
  end

  def self.configure(opts = {})
    opts.each do |key, val|
      @config[key.to_sym] = val if @config.keys.include?(key.to_sym)
    end
  end

  @cache = Cache.new

  @config = {
    :cache_on => false,
    :cache_expire => (1 * 60 * 60), # an hour
    :cache_location => "cache",
    :cache_storage_class => JobParser::Cache::TextFile
  }

end
