require 'mongoid'
module JobParser
  class Cache
    class MongoStore

      def has_cache_for_url?(url)
        job_for_url(url).count > 0
      end

      def store(hash)
        job_for_url(hash[:url]).delete
        hash = strip_fields_not_stored(hash)
        Job.create(hash)
      end

      def cache_expired?(url)
        job = job_for_url(url).first
        expire_time = (job.created_at + JobParser.config[:cache_expire])
        Time.now > expire_time
      end

      def get(url)
        job = job_for_url(url).first
        {}.tap do |job_obj|
          job.attributes.each do |k, v|
            job_obj[k.to_sym] = v unless %w{created_at _id updated_at}.include?(k)
          end
          job_obj[:from_cache] = true
        end
      end

      def clear_all
        MongoStore::Job.each(&:delete)
      end

      def view_cache
        [].tap do |res|
          Job.each do |job|
            res.push({
              :url => job.url,
              :created => job.created_at
            })
          end
        end
      end

      class Job
        include Mongoid::Document
        include Mongoid::Timestamps

        EXTRA_SCHEMA_TEXT_FIELDS = %w{
          benefits education_requirements incentives industry
          occupational_category qualifications responsibilities skills special_commitments work_hours
        }

        EXTRA_SCHEMA_TEXT_FIELDS.each do |f|
          field f.to_sym, :type => String
        end

        field :url,           :type => String
        field :salary,        :type => Array
        field :title,         :type => String
        field :apply,         :type => String
        field :salary_string, :type => String
        field :location,      :type => String
        field :deadline,      :type => String
        field :postcode,      :type => String
        field :schema,        :type => Boolean, :default => false
      end

      private

      def job_for_url(url)
        Job.where(:url => url)
      end

      def strip_fields_not_stored(hash)
        {}.tap do |new_hash|
          excluded_fields = [:from_cache]
          hash.each { |k, v| new_hash[k] = v unless excluded_fields.include?(k) }
        end
      end
    end
  end
end
