require 'digest/md5'
require 'json'
module JobParser
  class Cache
    class TextFile
      def has_cache_for_url?(url)
        File.exist?(path_for_url(url))
      end

      def store(job_hash)
        write_to_file(path_for_url(job_hash[:url]), job_hash.to_json)
      end

      def get(url)
        path = path_for_url(url)
        obj = JSON.parse(IO.read(path))
        sym_obj = make_object_keys_symbols(obj)
        sym_obj[:from_cache] = true
        sym_obj
      end

      def clear_all
        cache_files.each { |f| File.delete(f) }
      end

      def cache_expired?(url)
        time = File.mtime(path_for_url(url))
        expire_time = time + JobParser.config[:cache_expire]
        Time.now > expire_time
      end

      def view_cache
        [].tap do |res|
          cache_files.each do |f|
            contents = JSON.parse(IO.read(f))
            res.push({
              :url => contents["url"],
              :created => File.mtime(f)
            })
          end
        end
      end

      private

      def make_object_keys_symbols(obj)
        {}.tap do |sym_obj|
          obj.each { |k, v| sym_obj[k.to_sym] = v }
        end
      end

      def cache_files
        Dir[File.join(JobParser.config[:cache_location], "*.txt")]
      end

      def write_to_file(path, contents)
        File.open(path, "w") { |f| f.puts(contents) }
      end

      def path_for_url(url)
        cache_dir = JobParser.config[:cache_location]
        File.join(cache_dir, md5_url(url))
      end

      def md5_url(url)
        "#{Digest::MD5.hexdigest(url)}.txt"
      end
    end
  end
end
