module JobParser
  module Facets
    class Facet

      ACCEPTED_ELEMENTS = %w{p a h1 h2 h3 h4 h5 span dl dd dt td}

      attr_reader :doc, :url, :plain_text

      def initialize(doc, url, plain_text)
        @doc = doc
        @url = url
        @plain_text = plain_text
      end

      private

      def use_special_case(name)
        if special_case = SpecialCases.case_for_url(@url)
          unless special_case[name].nil?
            special_case[name].call(@doc)
          end
        end
      end

      def loop_over_elements(&block)
        elements.each do |name, elems|
          elems.each do |elem|
            yield name, elem
          end
        end
      end

      def elements
        {}.tap do |response|
          ACCEPTED_ELEMENTS.each do |elem|
            response[elem] = @doc.css(elem).to_a
          end
        end
      end

    end
  end
end
