require "nokogiri"
module JobParser
  class Parser
    ACCEPTED_ELEMENTS = %w{p a h1 h2 h3 h4 h5 span dl dd dt td}

    def initialize(html, from_url)
      @url = from_url
      @html = html
      @doc = strip_bad_elements(Nokogiri::HTML(@html))
      @plain_text = get_plain_text
    end

    def job
      if JobParser.cache.valid_for_url?(@url)
        JobParser.cache.fetch_result_for_url(@url)
      else
        { :url => @url,
          :salary => job_salary,
          :title => job_title,
          :apply => apply_link,
          :salary_string => job_salary_string,
          :location => job_location,
          :deadline => deadline,
          :postcode => job_postcode
        }
      end
    end

    private

    def facet_args
      [@doc, @url, @plain_text]
    end

    def cache(result)
      if JobParser.config[:cache_on]
        store_result_to_cache(result)
      end
      result
    end

    def store_result_to_cache(result)
      JobParser.cache.store_to_file(result)
    end

    def strip_bad_elements(doc)
      blacklist = ['script', 'style', 'button']
      blacklist.each do |tag|
        doc.xpath("//#{tag}").remove
      end
      doc.css("br").each { |br| br.replace "\n" }
      doc
    end

    def get_plain_text
      doc = @doc.dup
      blacklist = ['title', 'script', 'style', 'button']
      nodelist = doc.search('//text()')
      nodelist.text
    end
  end
end
