module JobParser
  class Scorer
    attr_reader :matches

    def initialize
      @matches = {}
    end

    def store(str, worth)
      match = nil
      if match = @matches[str]
        match = Match.new(str, worth, match.score)
      else
        match = Match.new(str, worth)
      end
      @matches[str] = match
      match
    end

    def score_for(str)
      @matches[str].nil? ? 0 : @matches[str].score
    end

    def top_match
      @matches.select { |k, v| v.score > 0 }.max_by { |k, v| v.score }.first
    end

    def store_and_score(str, worth)
      store(str, worth).and_score_now
    end

  end

  class Match
    attr_accessor :str, :worth
    attr_reader :score
    def initialize(str, worth, score = 0)
      @str = str
      @score = score
      @worth = worth
    end

    def if_regex_match(reg, str)
      reg.match(str) {
        @score += @worth
        true
      }
      false
    end

    def if_block_true(&block)
      res = yield
      @score += @worth if res
      res
    end

    def and_score_now
      @score += @worth
      self
    end
  end
end
